import os

from django.test import TestCase

from . import config, mixins
from ..backend import utils
from .. import settings as djangui_settings

class ScriptAdditionTests(mixins.ScriptFactoryMixin, TestCase):

    def test_command_order(self):
        script = os.path.join(config.DJANGUI_TEST_SCRIPTS, 'command_order.py')
        new_file = utils.get_storage(local=True).save(os.path.join(djangui_settings.DJANGUI_SCRIPT_DIR, 'command_order.py'), open(script))
        added, errors = utils.add_djangui_script(script=new_file, group=None)
        self.assertEqual(added, True, errors)
        job = utils.create_djangui_job({'djangui_type': 1, 'user': None, 'job_name': 'abc', 'link': 'alink', 'name': 'aname'})
        # These are positional arguments -- we DO NOT want them returning anything
        self.assertEqual(['', ''], [i.parameter.short_param for i in job.get_parameters()])
        # These are the params shown to the user, we want them returning their destination
        # This also checks that we maintain the expected order
        self.assertEqual(['link', 'name'], [i.parameter.script_param for i in job.get_parameters()])
        # Check the job command
        commands = utils.get_job_commands(job=job)[2:]
        self.assertEqual(['alink', 'aname'], commands)


import os

from django.test import TestCase

from . import factories, config, mixins

class ScriptTestCase(mixins.ScriptFactoryMixin, TestCase):

    def test_script_creation(self):
        script = factories.TranslateScriptFactory()


class ScriptGroupTestCase(TestCase):

    def test_script_group_creation(self):
        group = factories.ScriptGroupFactory()


class TestJob(mixins.ScriptFactoryMixin, TestCase):
    def test_jobs(self):
        script = factories.TranslateScriptFactory()
        from ..backend import utils
        from .. import settings
        # the test server doesn't have celery running
        settings.DJANGUI_CELERY = False
        job = utils.create_djangui_job({'djangui_type': script.pk, 'user': None, 'job_name': 'abc', 'sequence': 'aaa', 'out': 'abc'})
        job = job.submit_to_celery()
        old_pk = job.pk
        new_job = job.submit_to_celery(resubmit=True)
        self.assertNotEqual(old_pk, new_job.pk)
        # test job with a file
        job = utils.create_djangui_job({'djangui_type': script.pk, 'user': None, 'job_name': 'abc',
                                        'fasta': open(os.path.join(config.DJANGUI_TEST_DATA, 'fasta.fasta')),
                                        'out': 'abc'})


# copied from django-compressor since I like their style
import os
import django

TEST_DIR = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'tests')


CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'unique-snowflake'
    }
}

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'djangui',
]


STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
]

STATIC_URL = '/static/'


STATIC_ROOT = os.path.join(TEST_DIR, 'static')

if django.VERSION[:2] < (1, 6):
    TEST_RUNNER = 'discover_runner.DiscoverRunner'

SECRET_KEY = "iufoj=mibkpdz*%bob952x(%49rqgv8gg45k36kjcg76&-y5=!"

PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.UnsaltedMD5PasswordHasher',
)

MIDDLEWARE_CLASSES = []

ROOT_URLCONF = 'djangui.urls'


from django import get_version
from distutils.version import StrictVersion
DJANGO_VERSION = StrictVersion(get_version())
DJ18 = StrictVersion('1.8')
DJ17 = StrictVersion('1.7')
DJ16 = StrictVersion('1.6')


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import autoslug.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangui', '0003_auto_20150531_1650'),
    ]

    operations = [
        migrations.AlterField(
            model_name='djanguifile',
            name='filepath',
            field=models.FileField(upload_to='', max_length=500),
        ),
        migrations.AlterField(
            model_name='djanguijob',
            name='status',
            field=models.CharField(default='submitted', choices=[('submitted', 'Submitted'), ('running', 'Running'), ('completed', 'Completed'), ('deleted', 'Deleted')], max_length=255),
        ),
        migrations.AlterField(
            model_name='script',
            name='save_path',
            field=models.CharField(null=True, blank=True, max_length=255, help_text='By default save to the script name, this will change the output folder.'),
        ),
        migrations.AlterField(
            model_name='script',
            name='script_path',
            field=models.FileField(upload_to=''),
        ),
        migrations.AlterField(
            model_name='script',
            name='slug',
            field=autoslug.fields.AutoSlugField(unique=True, populate_from='script_name', editable=False),
        ),
        migrations.AlterField(
            model_name='scriptgroup',
            name='slug',
            field=autoslug.fields.AutoSlugField(unique=True, populate_from='group_name', editable=False),
        ),
        migrations.AlterField(
            model_name='scriptparameter',
            name='param_help',
            field=models.TextField(null=True, blank=True, verbose_name='help'),
        ),
        migrations.AlterField(
            model_name='scriptparameter',
            name='slug',
            field=autoslug.fields.AutoSlugField(unique=True, populate_from='script_param', editable=False),
        ),
        migrations.AlterField(
            model_name='scriptparameters',
            name='_value',
            field=models.TextField(db_column='value'),
        ),
    ]



from . import version
import os
if version.DJANGO_VERSION >= version.DJ17:
    default_app_config = 'djangui.apps.DjanguiConfig'
else:
    if os.environ.get('TESTING') != 'True':
        from . import settings as djangui_settings
        # we need to call from within djangui_settings so the celery/etc vars are setup
        if not djangui_settings.settings.configured:
            djangui_settings.settings.configure()
        from .apps import DjanguiConfig
        DjanguiConfig().ready()


import factory
import os
import six

from django.contrib.auth import get_user_model

from ..models import DjanguiJob, ScriptGroup, Script, ScriptParameter, ScriptParameterGroup, ScriptParameters

from . import config

class ScriptGroupFactory(factory.DjangoModelFactory):
    class Meta:
        model = ScriptGroup

    group_name = 'test group'
    group_description = 'test desc'

class ScriptFactory(factory.DjangoModelFactory):
    class Meta:
        model = Script

    script_name = 'test script'
    script_group = factory.SubFactory(ScriptGroupFactory)
    script_description = 'test script desc'

class TranslateScriptFactory(ScriptFactory):

    script_path = factory.django.FileField(from_path=os.path.join(config.DJANGUI_TEST_SCRIPTS, 'translate.py'))

class UserFactory(factory.DjangoModelFactory):
    class Meta:
        model = get_user_model()

    username = 'user'
    email = 'a@a.com'
    password = 'testuser'

class JobFactory(factory.DjangoModelFactory):
    class Meta:
        model = DjanguiJob

    script = factory.SubFactory(TranslateScriptFactory)
    job_name = six.u('\xd0\xb9\xd1\x86\xd1\x83')
    job_description = six.u('\xd0\xb9\xd1\x86\xd1\x83\xd0\xb5\xd0\xba\xd0\xb5')